--Auras
--[[
Drathals HUD2 (c) 2007 by Horacio Hoyos / Arcanefoam
Drathals HUD (c) 2006 by Caeryn Dryad
Drathals HUD (c) 2006 by Markus Inger / Drathal / Silberklinge / Silbersegen

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

    * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above 
       copyright notice, this list of conditions and the following 
       disclaimer in the documentation and/or other materials provided 
       with the distribution.
   * Neither the name of the DHUD2 Development Team nor the names of 
       its contributors may be used to endorse or promote products derived 
       from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]
local DHUD2, self = DHUD2, DHUD2

local math_floor = math.floor
local _G = _G
local auraUpdateInterval = 1
local formatTime
do
    --[[
            formatTime: transform time in seconds to days/hours/minutes
            arguments:
                secs: time in seconds
            returns:
                formtated time  --]]
    function formatTime(secs)
        if secs >= 86400 then
            return format('%dd', math_floor(secs/86400 + 0.5));
        elseif secs >= 3600 then
            return format('%dh', math_floor(secs/3600 + 0.5));
        elseif secs >= 60 then
            return format('%dm', math_floor(secs/60 + 0.5));
        else
            return (math_floor(secs + 0.5));
        end
    end
end

function DHUD2:updateCombos()
    local points = GetComboPoints()
    for i = 1, 5 do
        local combo = _G["DHUD2_combo"..i]
        if i <= points then
            combo:Show()
        else
            combo:Hide()
        end
    end
end
    
function DHUD2:updatePlayerPvP()
    local p = self.db.profile
    if p.show.playerPvpCcon and self.isAlive then
        local _, _, _, _, _, _, _, x0, x1, y0, y1, path = unpack(self.layout.icons.playerPvP)
        if UnitIsPVPFreeForAll("player")  then
            path = path[3]
        elseif UnitIsPVP("player") then 
            local faction = UnitFactionGroup("player")
            if faction == "Horde" then
                path = path[2]
            else
                path = path[1]
            end
        else
            DHUD2_playerPvP:Hide()
            return
        end
        local texture = _G["DHUD2_playerPvP_Texture"]
        texture:SetTexture(path)
        texture:SetTexCoord(x0,x1,y0,y1)
        DHUD2_playerPvP:Show()
    else
        DHUD2_playerPvP:Hide()
    end
end

function DHUD2:updatePlayerAuras()
    -- Auras can be buffs or Weapon Enchants
    local i, icon
    local j = 1
    local p = self.db.profile
    if p.show.playerBuffs then
        -- Buffs
        for i = 1, 40 do
            local buffI = GetPlayerBuff( i, "HELPFUL" )
            local timeLeft = GetPlayerBuffTimeLeft(buffI)            
            if timeLeft > 0 and timeLeft < p.gral.playerBuffTimeFilter then
                local r, g, b, hex = self:colorize("player", "aura", timeLeft / 20)
                local button = _G["DHUD2_playerBuff"..j]
                button.hasbuff = true
                button.hasweapon = false
                button.auraTimer = 0
                button.duration = math_floor(timeLeft)
                button.id = buffI
                button:SetNormalTexture(GetPlayerBuffTexture(buffI))
                local buffBorder = _G["DHUD2_playerBuff"..j.."_Border"]
                buffBorder:SetVertexColor(r, g, b)
                buffBorder:Show()
                local buffText = _G["DHUD2_playerBuff"..j.."_Text"]
                buffText:SetText("|cff"..hex..formatTime(button.duration))
                button:SetScript("OnUpdate", self.OnUpdateAura)
                button:EnableMouse(true)
                button:Show()
                if j == 16 then
                    self:Print("You have reached the buff limit. Contact Arcanefoam and ask for more buff slots")
                    break
                else
                    j = j + 1
                end
            end
        end
        -- Weapons
        local hasMainHandEnchant, mainHandExpiration, _, hasOffHandEnchant, offHandExpiration, _ = GetWeaponEnchantInfo()
        local id = 0
        local slot = 0
        for i = 1, 2 do
            -- Fit main and off hand
            if i == 1 and hasMainHandEnchant then
                timeLeft = mainHandExpiration/1000
                id = 1
                slot = 16
            elseif i == 2 and hasOffHandEnchant then
                timeLeft = offHandExpiration/1000
                id = 2
                slot = 17
            else
               timeLeft = 0 
            end
            if timeLeft > 0 and timeLeft < p.gral.playerBuffTimeFilter then
                local r, g, b, hex = self:colorize("player", "aura", timeLeft / 20)
                local button = _G["DHUD2_playerBuff"..j]
                button.hasbuff = false
                button.hasweapon = true
                button.auraTimer = 0
                button.duration = math_floor(timeLeft)
                button.id = id
                button:SetNormalTexture(GetInventoryItemTexture("player", slot))
                local buffBorder = _G["DHUD2_playerBuff"..j.."_Border"]
                buffBorder:SetVertexColor(r, g, b)
                buffBorder:Show()
                local buffText = _G["DHUD2_playerBuff"..j.."_Text"]
                buffText:SetText("|cff"..hex..formatTime(button.duration))
                button:SetScript("OnUpdate", self.OnUpdateAura)
                button:EnableMouse(true)
                button:Show()
                if j == 16 then
                    self:Print("You have reached the buff limit. Contact Arcanefoam and ask for more buff slots")
                    break
                else
                    j = j + 1
                end
            end
        end
    end
    for i = j, 16 do
        local button = _G["DHUD2_playerBuff"..i]
        button.id = 0
        button.hasbuff = false
        button.hasweapon = false
        button:SetScript("OnUpdate", nil)
        button:EnableMouse(false)
        button:Hide()
    end
end
    
function DHUD2:updateTargetAuras(enable)
    local i
    local buffBorder, buffText, buffFrame
    local debuffBorder, debuffText, debuffFrame
    local p = self.db.profile
    if not p.gral.swapTargetAuras then
        buffFrame   = "DHUD2_TRA";
        debuffFrame = "DHUD2_TLA";
    else
        debuffFrame = "DHUD2_TLA";
        buffFrame   = "DHUD2_TRA";    
    end
    local show = p.show.targetAuras and enable
    -- Buffs
    for i = 1, 40 do
        local button = _G[buffFrame..i]
        local buffName, _, buffTexture, buffApplication = UnitBuff("target", i)
        if show and buffName then
            button.hasbuff = true
            button.unit = "target"
            button.id = i
            --local icon = _G[button:GetName()]
            --icon:SetNormalTexture(buffTexture) 
            button:SetNormalTexture(buffTexture)
            buffBorder = _G[buffFrame..i.."_Border"]
            buffBorder:Hide()
            buffText = _G[buffFrame..i.."_Text"]
            if buffApplication <= 1 then
                buffText:SetText("")
            elseif buffApplication > 1 then
                buffText:SetText(buffApplication)
            end
            button:Show()
        else    
            button:Hide()
            button.hasdebuff = false
        end
    end        
    -- DeBuffs
    for i = 1, 40 do
        local button = _G[debuffFrame..i]
        local debuffName, _, debuffTexture, debuffApplication = UnitDebuff("target", i)
        if show and debuffName then
            button.hasdebuff = true
            button.unit = "target"
            button.id = i
            --local icon = _G[button:GetName()]
            --icon:SetNormalTexture(debuffTexture)
            button:SetNormalTexture(debuffTexture)
            debuffBorder = _G[debuffFrame..i.."_Border"]
            debuffBorder:Show()
            debuffText = _G[debuffFrame..i.."_Text"]
            if debuffApplication <= 1 then
                debuffText:SetText("")
            elseif debuffApplication > 1 then
                debuffText:SetText(debuffApplication)
            end
            button:Show()
        else
            button:Hide()
            button.hasdebuff = false
        end
    end
end

function DHUD2:updateTargetIcon()    
    local p = self.db.profile
    if p.show.targetPvpIcon then
        local _, _, _, _, _, _, _, x0, x1, y0, y1, path = unpack(self.layout.icons.playerPvP)
        if UnitIsPVPFreeForAll("target")  then
            path = path[3]
        elseif UnitIsPVP("target") then 
            local faction = UnitFactionGroup("target")
            if faction == "Horde" then
                path = path[2]           
            else
                path = path[1]                
            end
        else
            local index = GetRaidTargetIndex("target")
            if index then
                if index > 0 and index <= 8 then
                    path, x0, x1, y0, y1 = unpack( self.layout.raidIcons[index] )
                end
            end
        end
        local texture = _G["DHUD2_targetIcon_Texture"]
        texture:SetTexture(path)
        texture:SetTexCoord(x0,x1,y0,y1)
        DHUD2_targetIcon:Show()
    else
        DHUD2_targetIcon:Hide()
    end
end

function DHUD2:updatePetIcon(health, happiness)
    local p = self.db.profile
    --self:Debug("updatePetIcon", health, happiness)
    if health > 0 and happiness then
        local _, texture, x0, x1, y0, y1 = unpack( self.layout.images.pet[happiness] )
        if texture then
            local tex = _G["DHUD2_petHappy_Texture"]
            tex:SetTexture(texture)
            tex:SetTexCoord(x0, x1, y0, y1)
            DHUD2_petHappy:Show()
        else
            DHUD2_petHappy:Hide()
        end
    else
        DHUD2_petHappy:Hide()
    end
end
    
function DHUD2:OnUpdateAura(elapsed)
    self.auraTimer = self.auraTimer + elapsed
    if self:GetEffectiveAlpha() == 0 then
        self:EnableMouse(false)
    else            
        self:EnableMouse(true)
        if self.auraTimer >= auraUpdateInterval then
            self.auraTimer = 0
            local timeLeft
            local slot
            if self.hasbuff then 
                timeLeft = math_floor(GetPlayerBuffTimeLeft(self.id))
            elseif self.hasweapon then
                local _, mainHandExpiration, _, _, offHandExpiration, _ = GetWeaponEnchantInfo()
                if self.id == 1 then 
                    timeLeft = mainHandExpiration/1000
                    slot = 16
                elseif self.id == 2 then
                    timeLeft = offHandExpiration/1000
                    slot = 17
                end
            end
            self.duration = timeLeft           
            local r, g, b, hex = DHUD2:colorize("player", "aura", self.duration / 20)
            local buffBorder = _G[self:GetName().."_Border"]
            buffBorder:SetVertexColor(r, g, b)
            local buffText = _G[self:GetName().."_Text"]
            buffText:SetText("|cff"..hex..formatTime(self.duration))
        end


    end
end

function DHUD2:updateTargetElite()
    local p = self.db.profile
    if self:checkElite("target") and p.show.eliteIcon then
        local images = self.layout.icons
        local _, _, _, _, _, _, _, _, _, _, _, texture = unpack(images.targetElite)
        self:Debug("updateTargetElite", texture)
        DHUD2_targetElite_Texture:SetTexture(texture)
        DHUD2_targetElite:Show()
    else
        DHUD2_targetElite:Hide()
    end
end
    